/**
  ******************************************************************************
  * @file    main.c

  ******************************************************************************
*/

#include <stdlib.h>
#include "stm32f4xx.h"
#include "stm32f4xx_hal.h"
#include "stm32f4xx_hal_uart.h"
#include "stm32f429i_discovery.h"
#include "stm32f429i_discovery_lcd.h"
#include "system_clocks.h"
#include "simpleLCD.h"
#include "onewire.h"
#include "serial.h"

/* Private typedef -----------------------------------------------------------*/

/* Exported macro ------------------------------------------------------------*/

/* Private macro -----------------------
 * --------------------------------------*/
/* Private variables ---------------------------------------------------------*/
/* UART handler declaration */
UART_HandleTypeDef UartHandle;

static char str[20];

/* Private function prototypes -----------------------------------------------*/
//static void Error_Handler(void);

/* Private functions ---------------------------------------------------------*/

int main(void)
{
  uint8_t stan1wire = 0;
  uint8_t daneROM[8]={0};

  HAL_Init();

  /* Configure LED3 and LED4 */
  BSP_LED_Init(LED3);
  BSP_LED_Init(LED4);

  /* Configure the system clock to 180 MHz */
  SystemClock_Config();

  Serial_Init();
  OneWire_Init();
  LCD_Init();
  LCD_SetFont(FONT16);
  LCD_PrintXY(5,1,"Witam 1-wire!");
  BSP_LED_On(LED3);
  BSP_LED_Off(LED4);

  /* Infinite loop */
  while(1){
    BSP_LED_Toggle(LED3); //czyli PG13
    BSP_LED_Toggle(LED4);
    while(stan1wire){
    	LCD_PrintXY(1,11,"                             ");
    	LCD_PrintXY(1,17," Czekam na 1wire... ");
    	delayUS(10);
    	stan1wire = OneWire_Reset();
    }
    LCD_PrintXY(1,17," 1wire... OK!     ");

    stan1wire = OneWire_GetROM(daneROM);
    if(stan1wire) continue; // błąd lub brak urządzenia
    LCD_PrintXY(1,3,"Dane z ROM");
    sprintf(str,"Family code : 0x%X  ",daneROM[0]);
    LCD_PrintXY(1,4,str);
    LCD_PrintXY(1,6,"Numer seryjny ROM:");
    sprintf(str,"0x%02X%02X%02X%02X%02X%02X ",daneROM[1],daneROM[2],daneROM[3],daneROM[4],daneROM[5],daneROM[6]);
    LCD_PrintXY(1,7,str);
    sprintf(str,"CRC : 0x%X  ",daneROM[7]);
    LCD_PrintXY(1,9,str);

    if(stan1wire) continue; // błąd lub brak urządzenia
    if(daneROM[0]==0x28){
      LCD_PrintXY(1,11,"Termometr DS18B20 OK ");
	  LCD_PrintXY(1,13,"Pomiar :");
	  float temp = OneWire_GetTemperature();
      sprintf(str,"Temp = %4.2fC  ",temp);
	  LCD_PrintXY(1,14,str);
	  sprintf(str,"Temp=%6.2fC\n", temp);
	  Serial_Send((uint8_t*)str, 13);
    }
    else if(daneROM[0]==0x07)
    {
    	LCD_PrintXY(1,10,"Dostepny czytnik RFID       ");
    	sprintf(str,"Code=0x%02X,SN=0x%02X%02X%02X%02X%02X%02X,CRC=%02X\n ",daneROM[0],daneROM[1],
    			daneROM[2],daneROM[3],daneROM[4],daneROM[5],daneROM[6],daneROM[7]);
    	Serial_Send((uint8_t*)str, 35);
    }
  }
}
