/*
 * J.Piwowarczyk
 * materiały dydaktyczne
 *
 */

#include <stdlib.h>
#include "stm32f4xx_ll_gpio.h"
#include "stm32f4xx.h"
#include "stm32f429i_discovery.h"
#include "stm32f429i_discovery_lcd.h"
#include "system_clocks.h"
#include "onewire.h"

#define SC1M (SystemCoreClock/1000000)

inline void  delayUS(volatile uint32_t usek){
	uint32_t tmp = (usek<2)?(5*usek):(SC1M *usek*125) / 1000;//dzielnik;
	while (tmp--);
}

void OneWire_Init(void){
  GPIO_InitTypeDef  GPIO_InitStruct;
  //LL_GPIO_InitTypeDef GPIO_InitStruct;

  /* Enable GPIOA clock */
  // HAL
    __HAL_RCC_GPIOC_CLK_ENABLE();
  // LL
  //LL_AHB1_GRP1_EnableClock(LL_AHB1_GRP1_PERIPH_GPIOF);
  /* Configure GPIO pin : OneWire_Pin */
  /* Configure  pin as output */
  // HAL
  GPIO_InitStruct.Pin = OneWire_Pin;
  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
  GPIO_InitStruct.Pull = GPIO_PULLUP;
  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
  HAL_GPIO_Init(OneWire_Port, &GPIO_InitStruct);
  HAL_GPIO_WritePin(OneWire_Port, OneWire_Pin, GPIO_PIN_SET);
  // LL
  /**/
//    GPIO_InitStruct.Pin = OneWire_Pin;
//    GPIO_InitStruct.Mode = LL_GPIO_MODE_OUTPUT;
//    GPIO_InitStruct.Speed = LL_GPIO_SPEED_FREQ_HIGH;
//    GPIO_InitStruct.OutputType = LL_GPIO_OUTPUT_PUSHPULL;
//    GPIO_InitStruct.Pull = LL_GPIO_PULL_UP;
//    LL_GPIO_Init(OneWire_Port, &GPIO_InitStruct);
}

void inline OneWire_SetInput(void){
  //GPIO_InitTypeDef  GPIO_InitStruct;

  /*Configure GPIO pin : OneWire_Pin */
  /* Configure  pin as input  */
//  GPIO_InitStruct.Pin = OneWire_Pin;
//  GPIO_InitStruct.Mode = GPIO_MODE_INPUT
//  GPIO_InitStruct.Pull = GPIO_PULLUP;
//  HAL_GPIO_Init(OneWire_Port, &GPIO_InitStruct);
  // LL
  LL_GPIO_SetPinMode(OneWire_Port,OneWire_Pin,LL_GPIO_MODE_INPUT);
  LL_GPIO_SetPinPull(OneWire_Port,OneWire_Pin,LL_GPIO_PULL_UP);
}

void inline OneWire_SetOutput(void){
  //GPIO_InitTypeDef  GPIO_InitStruct;

  /*Configure GPIO pin : OneWire_Pin */
  /* Configure PC8 pin as output */
//  GPIO_InitStruct.Pin = OneWire_Pin;
//  GPIO_InitStruct.Mode = GPIO_MODE_OUTPUT_PP;
//  GPIO_InitStruct.Pull = GPIO_PULLUP;
//  GPIO_InitStruct.Speed = GPIO_SPEED_FREQ_HIGH;
//  HAL_GPIO_Init(OneWire_Port, &GPIO_InitStruct);
  // LL
  LL_GPIO_SetPinMode(OneWire_Port,OneWire_Pin,LL_GPIO_MODE_OUTPUT);
  LL_GPIO_SetPinOutputType(OneWire_Port,OneWire_Pin,LL_GPIO_OUTPUT_PUSHPULL);
  LL_GPIO_SetPinPull(OneWire_Port,OneWire_Pin,LL_GPIO_PULL_UP);
}

inline void  OneWire_SetBus(void){
	//HAL_GPIO_WritePin(OneWire_Port, OneWire_Pin, GPIO_PIN_SET);
	LL_GPIO_SetOutputPin(OneWire_Port, OneWire_Pin);
}

inline void  OneWire_ResetBus(void){
	//HAL_GPIO_WritePin(OneWire_Port, OneWire_Pin, GPIO_PIN_RESET);
	LL_GPIO_ResetOutputPin(OneWire_Port, OneWire_Pin);
}

OneWireState  OneWire_GetBus(void){
	// HAL
	return (OneWireState)HAL_GPIO_ReadPin(OneWire_Port, OneWire_Pin);
	// LL
	//return (OneWireState)LL_GPIO_IsOutputPinSet(OneWire_Port, (uint32_t)OneWire_Pin);
}

uint8_t OneWire_Reset(void){
  uint8_t stan = 0;
  OneWireState oneState = 0;

  //OneWire_SetOutput();
  OneWire_SetBus();
  OneWire_SetInput();
  delayUS(2);
  oneState = OneWire_GetBus();
  if( !oneState ) return (1);
  OneWire_SetOutput();
  OneWire_ResetBus();
  delayUS(480);
  OneWire_SetInput();
  delayUS(60);
  stan = (OneWire_GetBus())?1:0;
  delayUS(480);
  //OneWire_SetOutput();
  OneWire_SetBus();
  return stan;
}

void OneWire_WriteBit(uint8_t bit){
	OneWire_SetOutput();
	OneWire_ResetBus();
	delayUS(5);
	if(bit) OneWire_SetBus();
	delayUS(56);
	OneWire_SetBus();
}

uint8_t OneWire_ReadBit(void){
	uint8_t dana = 0;

	OneWire_SetOutput();
	//delayUS(2);
    OneWire_ResetBus();
	delayUS(3);
	//OneWire_SetBus();
	OneWire_SetInput();
	delayUS(7);
	if( OneWire_GetBus()) dana = 1;
	delayUS(50);
	OneWire_SetOutput();
	OneWire_SetBus();
	return dana;
}

void OneWire_WriteByte(uint8_t byt){
	uint8_t dana=0;
	__disable_irq();
	for(uint8_t i=0; i<8;++i){
		dana=(((1<<i)&byt)>>i);
		OneWire_WriteBit(dana);
	}
	__enable_irq();
}

uint8_t OneWire_ReadByte(void){
	__disable_irq();
	uint8_t dana=0, wynik=0;
	for(uint8_t i=0; i<8;++i){
		dana=OneWire_ReadBit();
		wynik|=(dana)?(1<<i):0;
	}
	__enable_irq();
	return wynik;
}

float OneWire_GetTemperature(){
	float wynik=0.0;
	uint8_t stan1wire = 0;
	uint8_t dane1wire[9]={0};
	uint16_t dana=0, reszta=0;

	stan1wire = OneWire_Reset();
	OneWire_WriteByte(0xCC); // skip ROM
    OneWire_WriteByte(0x44); // start conversion
	while(!(stan1wire=OneWire_ReadBit()) ) HAL_Delay(20);
	stan1wire = OneWire_Reset();
	OneWire_WriteByte(0xCC); // skip ROM
	OneWire_WriteByte(0xBE); // read 1wire
	for(uint8_t k=0;k<9;++k) dane1wire[k]=OneWire_ReadByte();
	dana=( ((uint16_t)dane1wire[1])<<8) | (uint16_t)dane1wire[0];
	reszta=(uint16_t)(((dana & 0x0f))*625);
	dana>>=4; // część całkowita

	wynik=(float)dana+(float)(reszta)/10000.0 ;
	return wynik;
}

uint8_t  OneWire_GetROM(uint8_t* daneROM){
	uint8_t stan1wire = 0;
    stan1wire = OneWire_Reset();
    if(stan1wire) return 1; //??
    OneWire_WriteByte(0x33); //read ROM commamd, skip ROM id
    for(uint8_t k=0;k<8;++k) daneROM[k]=OneWire_ReadByte();
    return 0;
}

void OneWire_Set10BitsRes(void){
	uint8_t dane1wire[9]={0};
	uint8_t k=0;
	uint8_t stan1wire=0 ;
	stan1wire = OneWire_Reset();
	//if(!stan1wire) return; //??
	OneWire_WriteByte(0xCC); // skip ROM
	OneWire_WriteByte(0xBE); // read 1wire
	for(k=0;k<9;++k) dane1wire[k]=OneWire_ReadByte();
	dane1wire[4]=0x3f; // 10 bitów rozdzielczość
	stan1wire = OneWire_Reset();
	//if(!stan1wire) return; //??
	OneWire_WriteByte(0xCC); // skip ROM
	OneWire_WriteByte(0x4E); // write 1wire
	for(k=2;k<5;++k) OneWire_WriteByte(dane1wire[k]);
}
